@@ -1,14 +1,14 @@
 ;;; visual-fill-column.el --- fill-column for visual-line-mode  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015-2020 Joost Kremers
+;; Copyright (C) 2015-2025 Joost Kremers
 ;; Copyright (C) 2016 Martin Rudalics
 ;; All rights reserved.
 
 ;; Author: Joost Kremers <joostkremers@fastmail.fm>
 ;; Maintainer: Joost Kremers <joostkremers@fastmail.fm>
-;; URL: https://github.com/joostkremers/visual-fill-column
+;; URL: https://codeberg.org/joostkremers/visual-fill-column
 ;; Created: 2015
-;; Version: 2.4
+;; Version: 2.7.1
 ;; Package-Requires: ((emacs "25.1"))
 
 ;; This file is NOT part of GNU Emacs.
@@ -31,12 +31,18 @@
 ;; `visual-fill-column-mode' is a small Emacs minor mode that mimics the effect
 ;; of `fill-column' in `visual-line-mode'.  Instead of wrapping lines at the
 ;; window edge, which is the standard behaviour of `visual-line-mode', it wraps
-;; lines at `fill-column'.  If `fill-column' is too large for the window, the
-;; text is wrapped at the window edge.
+;; lines at `fill-column' (or `visual-fill-column-width', if set).  This is
+;; accomplished by widening the margins, which narrows the text area.  When the
+;; window size changes, the margins are adjusted automatically.
+;;
+;; When `visual-fill-column-center-text' is set, the text is centered in the
+;; window.  This can also be used in combination with `auto-fill-mode' instead
+;; of `visual-line-mode', or in programming modes.
+
 
 ;;; Code:
 
-(defgroup visual-fill-column nil "Wrap lines according to `fill-column' in `visual-line-mode'."
+(defgroup visual-fill-column nil "Wrap lines according to `fill-column'."
   :group 'text
   :prefix "visual-fill-column-")
 
@@ -89,6 +95,15 @@ If this option is set, `visual-fill-column' sets the variable
   :type '(choice (const :tag "Allow vertical window split" nil)
                  (const :tag "Use standard window split" t)))
 
+(defcustom visual-fill-column-adjust-for-text-scale t
+  "Adjust the margins for text scaling.
+If set to t, the calculated margins are additionally adjusted for
+the text scale factor, so that the text is wrapped at
+`fill-column'."
+  :group 'visual-fill-column
+  :type '(choice (const :tag "Adjust margins for text scaling" t)
+                 (const :tag "Do not adjust margins for text scaling" nil)))
+
 (defvar visual-fill-column--use-split-window-parameter nil "If set, the window parameter `split-window' is used.")
 
 (defvar visual-fill-column--use-min-margins nil "If set, the window parameter `min-margins' is used.")
@@ -128,7 +143,15 @@ If this option is set, `visual-fill-column' sets the variable
 
 ;;;###autoload
 (define-minor-mode visual-fill-column-mode
-  "Wrap lines according to `fill-column' in `visual-line-mode'."
+  "Soft-wrap lines according to `fill-column'.
+This minor mode narrows the text area.  Its primary use is in
+conjunction with `visual-line-mode', to enable soft word-wrapping of
+long lines, but it can also be used in other contexts, e.g., to center
+the text in a window.
+
+Note: to activate and deactivate `visual-fill-column-mode' together with
+`visual-line-mode', add the function `visual-fill-column' to
+`visual-line-mode-hook'."
   :init-value nil :lighter nil :global nil
   (if visual-fill-column-mode
       (visual-fill-column-mode--enable)
@@ -139,10 +162,39 @@ If this option is set, `visual-fill-column' sets the variable
   :require 'visual-fill-column-mode
   :group 'visual-fill-column)
 
+;;;###autoload
+(defun visual-fill-column-for-vline ()
+  "Hook function for `visual-line-mode-hook'.
+Use this function to enable and disable `visual-fill-column-mode' in
+conjunction with `visual-line-mode'."
+  (if visual-line-mode
+      (visual-fill-column-mode +1)
+    (visual-fill-column-mode -1)))
+
+;;;###autoload
+(define-minor-mode visual-line-fill-column-mode
+  "Enable `visual-line-mode' and soft-wrap lines according to `fill-column'."
+  :init-value nil :lighter nil :global nil
+  (cond (visual-line-fill-column-mode
+         (visual-fill-column-mode 1)
+         (visual-line-mode 1))
+        (t
+         (visual-fill-column-mode -1)
+         (visual-line-mode -1))))
+
+(make-obsolete 'visual-line-fill-column-mode
+               "add `visual-fill-column-for-vline' to `visual-line-mode-hook' instead."
+               "2.7.0")
+
+(defun visual-fill-column-toggle-center-text ()
+  "Toggle centering of text in the current buffer."
+  (interactive)
+  (setq visual-fill-column-center-text (not visual-fill-column-center-text))
+  (visual-fill-column-adjust))
+
 (defun turn-on-visual-fill-column-mode ()
   "Turn on `visual-fill-column-mode'.
 Note that `visual-fill-column-mode' is only turned on in buffers
-in which Visual Line mode is active as well, and only in buffers
 that actually visit a file."
   (when buffer-file-name
     (visual-fill-column-mode 1)))
@@ -160,11 +212,12 @@ that actually visit a file."
     (setq-default split-window-preferred-function #'visual-fill-column-split-window-sensibly))
 
   (cond
-   ((version<= emacs-version "27.1")
+   ((< emacs-major-version 27)
+    (add-hook 'window-size-change-functions #'visual-fill-column--adjust-frame))
+   ((= emacs-major-version 27)
     (add-hook 'window-size-change-functions #'visual-fill-column--adjust-window 'append 'local)
     (setq visual-fill-column--use-split-window-parameter t))
-
-   ((version< "27.1" emacs-version)
+   ((> emacs-major-version 27)
     (add-hook 'window-state-change-functions #'visual-fill-column--adjust-window 'append 'local)
     (setq visual-fill-column--use-min-margins t)))
 
@@ -176,13 +229,14 @@ that actually visit a file."
 
   (let ((window (get-buffer-window (current-buffer))))
     (cond
-     ((version<= emacs-version "27.1")
+     ((< emacs-major-version 27)
+      (remove-hook 'window-size-change-functions #'visual-fill-column--adjust-frame))
+     ((= emacs-major-version 27)
       (remove-hook 'window-size-change-functions #'visual-fill-column--adjust-window 'local))
-
-     ((version< "27.1" emacs-version)
+     ((> emacs-major-version 27)
       (remove-hook 'window-state-change-functions #'visual-fill-column--adjust-window 'local)
-      (set-window-margins window 0 0)
       (set-window-parameter window 'min-margins nil)))
+    (set-window-margins window 0 0)
     (set-window-fringes window nil)))
 
 (defun visual-fill-column-split-window (&optional window size side)
@@ -249,6 +303,14 @@ selected window has `visual-fill-column-mode' enabled."
           (set-window-parameter window 'min-margins '(0 . 0)))
       (visual-fill-column--set-margins window))))
 
+(defun visual-fill-column--adjust-frame (frame)
+  "Adjust the windows on FRAME.
+This function is added to `window-size-change-functions' in older
+Emacsen (before 27.1), in which the functions in this hook are
+passed the frame as argument."
+  (dolist (window (window-list frame))
+    (visual-fill-column--adjust-window window)))
+
 (defun visual-fill-column-adjust (&optional _inc)
   "Adjust the window margins and fringes.
 This function is for use as advice to `text-scale-adjust'.  It
@@ -267,22 +329,27 @@ and `text-scale-mode-step'."
   (or window (setq window (selected-window)))
   (let* ((margins (window-margins window))
          (buffer (window-buffer window))
-         (scale (if (and (boundp 'text-scale-mode-step)
+         (scale (if (and visual-fill-column-adjust-for-text-scale
+                         (boundp 'text-scale-mode-step)
                          (boundp 'text-scale-mode-amount))
                     (with-current-buffer buffer
                       (expt text-scale-mode-step
                             text-scale-mode-amount))
-                  1.0)))
-    (truncate (/ (+ (window-width window)
-                    (or (car margins) 0)
-                    (or (cdr margins) 0))
+                  1.0))
+         (remap-scale
+          (if (>= emacs-major-version 29)
+              (/ (window-width window 'remap) (float (window-width window)))
+            1.0)))
+    (truncate (/ (+ (window-width window (and (>= emacs-major-version 29) 'remap))
+                    (* (or (car margins) 0) remap-scale)
+                    (* (or (cdr margins) 0) remap-scale))
                  (float scale)))))
 
 (defun visual-fill-column--add-extra-width (left right add-width)
   "Calculate new margins given additional text width.
 LEFT and RIGHT are the current margins, ADD-WIDTH a cons cell of
-  additional columns to be added to the text area.  Return a cons
-  cell of the new margins, which will never be less than zero."
+additional columns to be added to the text area.  Return a cons
+cell of the new margins, which will never be less than zero."
   (cons (max 0 (- left (car add-width)))
         (max 0 (- right (cdr add-width)))))
 
@@ -290,11 +357,15 @@ LEFT and RIGHT are the current margins, ADD-WIDTH a cons cell of
   "Set window margins for WINDOW."
   ;; Calculate left & right margins.
   (let* ((total-width (visual-fill-column--window-max-text-width window))
+         (remap-scale
+          (if (>= emacs-major-version 29)
+              (/ (window-width window 'remap) (float (window-width window)))
+            1.0))
          (width (or visual-fill-column-width
                     fill-column))
          (margins (if (< (- total-width width) 0) ; margins must be >= 0
                       0
-                    (- total-width width)))
+                    (round (/ (- total-width width) remap-scale))))
          (left (if visual-fill-column-center-text
                    (/ margins 2)
                  0))
